%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%    MASTER CODE    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear

% Random number seed to enable replication
stream=RandStream.create('mrg32k3a','Seed',13);
RandStream.setDefaultStream(stream);
stream.Substream=86;
repeatstate1=stream.State;
stream.Substream=35;
repeatstate2=stream.State;
stream.Substream=4;
repeatstate3=stream.State;

MRUNS=30;
rstate=[1 5 18 69 74 20 49 11 56 1009 23 47 58 13 85 52 29 85 46 6 99 216 549 876 316 545 468 736 984 2546];
parmfit=zeros(MRUNS,7);

VARLAYER=zeros(80,80);
%%
for mrun=1:MRUNS
    %%
    stream.Substream=mrun;

%<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
    % M-files needed to run
    % 1. EmpDataInput.m
    % 2. LandMarket_6v0.m
    % 3. HouseMarketInitial.m
    % 4. HousemarketDynamic.m
    % 5. CHALMS_base_v1_0.m
    % 6. RefLandscape_CHALMS_base_v1_0_zone0
    % 7. BrokersModule_8v0_pop.m
    % 8. FarmerModule_7v0_pop.m
%<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

    %%% Housing Layer %%%
    HTYPE=3;
    LTYPE=6;
        
%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
%@@@@@@@@@@@@@@@@@@@@    INITIAL CONDITIONS    @@@@@@@@@@@@@@@@@@@@@@@@@@@@
%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    
    EmpDataInput
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%    PARAMETERS    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    DELTA=0.0527;
    survivalrate=0.9499;
    LOCWGHT=0.2955;
    REGWGHT=1-LOCWGHT;
    PCTSEARCH=0.5887;
    zeta=0.5;
  
    %Time settings
    TMAX=30;
    TSTART=10;
    
    %%% Landscape Layer %%%
    NWIDTH=80;
    NLENGTH=80;
    mile2acre=640;                          %1 sq mile = 640 acres
    cellside=1;   %linear distance
    cell2mile=0.0395;   %cell side equals 0.0395 mi
    celldiag=(cellside*sqrt(2));          %miles diag to neighboring cell's center
    acre2sqft=43560;
    avgdist2nei=mean([cellside celldiag]);
    margtc=1.30*500*cell2mile;        %Assumed: 250 travel days a year, roundtrip
    
    %%% Zones Layer %%%
    NZONES=25;
    
    %%% Broker Layer %%%
    Nbrokers=(NLENGTH/5)^2;
    
    %%% Agricultural Layer %%%
    Nfarmers=50;
    
    %%% Price Projection Models %%%
    NUMMODEL=20;
    FARMNUMCLASS=6;
    POPNUMCLASS=5;
    BROKERNUMCLASS=6;
    MAXMEANMODEL=10;
    MAXCYCLEMODEL=10;
    MAXPROJECT=10;
    
    %%% Farmers %%%
    FARMPROD=450;
    FARMCOST=225;
    PRODSTD=20;
    COSTSTD=50;
    % distance models for farmers
    NUMMODELDIST=100;
    maxcoeff=200;
    mincoeff=-200;
    % survivalrate=0.1;
    selectivity=NUMMODELDIST*survivalrate;
    nextgen=10;
    recombo=40;
    
    %%% Developer %%%
    Ndevelopers=1;
    inmaxrent=10000;
    CLOSECELL=30;
   
    
    %%% Consumers %%%
    maxwage=max(incomedata);
    % maxwage=200000;
    minwage=min(incomedata);
    % minwage=20000;
    himaxwage=maxwage;
    himinwage=100000;
    midmaxwage=99999;
    midminwage=60000;
    lowmaxwage=59999;
    lowminwage=minwage;
    WAGECLASS=3;
    HIBETA=[0.16 0.24];     %approximate max and mins for housepref come from calvert county census data
    MIDBETA=[0.25 0.33];
    LOWBETA=[0.34 0.42];
    incomeg=0.05;
    incomesigma=1;
    incomegrow=0.005;
    
    %%% Population %%%
    popg=0.05;
    popsigma=1;
    pop2dem=2.91;   %persons per household, Calvert County quickfacts
    POPGROW=0.10;   %ditto (19% actually)
    popurb=0.7;
    popag=0.3;
    
    % zeta=0.50;   %dampening on epsilon-- must be >0, controls the rate of price ...
    %increase/preception of market power
    
    discount=0.05;
    tax=0.05;
    thetavac=0.2;
    thetadev=0.2;
    
    %Flags
    epflag=0;
    oldbuildflag=0;
    newbuildflag=0;
    termrunflag=0;
    
    
    testtime=[10 15 20 25 30];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%    VARIABLES    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%% Landscape Layer %%%
    BASELAYER=zeros(NLENGTH,NWIDTH);  %developed or undeveloped
    DISTANCE=ones(NLENGTH,NWIDTH);
    dist2hznnei=zeros(NLENGTH,NWIDTH);    %distance to horizontal neighbor from icenter
    dist2vrtnei=zeros(NLENGTH,NWIDTH);
    travelcost=zeros(NLENGTH,NWIDTH);
    
    %%% Zones Layer %%%
    ZONES=zeros(NLENGTH,NWIDTH);
    zoning=zeros(NZONES,2);     %[(min lotsize) (max lotsize)]
    ZONEMAP=zeros(NLENGTH,NWIDTH);
    %%% Housing Layer %%%
    HOUSESIZE=zeros(NLENGTH,NWIDTH);
    LOTS=zeros(NLENGTH,NWIDTH);
    LOTTYPE=zeros(NLENGTH,NWIDTH);
    
    %%% Broker Layer %%%
    HBROKER=zeros(NLENGTH,NWIDTH);
    UUcell=zeros(NLENGTH,NWIDTH);
    
    %%% Agricultural Layer %%%
    AGLAYER=zeros(NLENGTH,NWIDTH);
    LANDVALUE=zeros(NLENGTH,NWIDTH);
    PLAND=zeros(NLENGTH,NWIDTH);
    
    %%% Farmers %%%
    Farmstats=zeros(Nfarmers,5,TMAX);    %[id acres prod costs value_acre]
    iNfarmers=(1:Nfarmers)';
    wtaland=zeros(Nfarmers,TMAX);
    Paskland=zeros(Nfarmers,TMAX);
    transprice=zeros(Nfarmers,TMAX);
    Plandproj=zeros(Nfarmers,TMAX);
    maxrent=zeros(Nfarmers,TMAX);
    maxreturn=zeros(Nfarmers,TMAX);
    landsold=zeros(NZONES,TMAX);
    sellrecord=zeros(Nfarmers,1);
    pdrrecord=zeros(Nfarmers,1);
    buyrecord=zeros(Nfarmers,1);
    
    fitness=zeros(Nfarmers,NUMMODELDIST,TMAX);
    
     % Price Projection models
    landproj = zeros(Nfarmers,NUMMODEL);
    landerror = zeros(Nfarmers,NUMMODEL);
    landbestSAVE=zeros(Nfarmers,TMAX);
    ilandbestSAVE=zeros(Nfarmers,TMAX);
    landprojSAVE=zeros(Nfarmers,TMAX);
    landprojbestSAVE=zeros(Nfarmers,NUMMODEL);
    landmodelbestSAVE=zeros(Nfarmers,TMAX);
    landmodelSAVE=zeros(Nfarmers,TMAX);
    
    %%% Developer %%%
    Profit=zeros([],1);
    Rmin=zeros([],1);      %minimum rent
    Paskhouse=zeros([],1);
    Ccost=zeros([],1);
    Pland=zeros([],1);
    Plot=zeros([],1);
    INRENT=zeros(NLENGTH,NWIDTH);
    BUILDTIME=zeros(NLENGTH,NWIDTH);
    PROFIT=zeros(NLENGTH,NWIDTH,TMAX);
    RETURN=zeros(NLENGTH,NWIDTH,LTYPE*HTYPE);
    RETURNPROJ=zeros(NLENGTH,NWIDTH);
    LOTPRICE=zeros(NLENGTH,NWIDTH);
    LOTVALUE=zeros(NLENGTH,NWIDTH);
    INITIALPASK=zeros(NLENGTH,NWIDTH);
    RENT=zeros(NLENGTH,NWIDTH);
    house2cell=zeros(NLENGTH,NWIDTH);
    RENTGRAD=zeros(NLENGTH,NWIDTH);
    XBIDS=zeros(NLENGTH,NWIDTH);
    VACLAND=zeros(NLENGTH,NWIDTH);
    FARMPROJ=zeros(NLENGTH,NWIDTH);
    RENTPROJ=zeros(NLENGTH,NWIDTH);
    RENTPROJLAND=zeros(NLENGTH,NWIDTH);
    maxRENTPROJ=zeros(NLENGTH,NWIDTH);
    CCOST=zeros(NLENGTH,NWIDTH);
    ZZ=zeros(NLENGTH,NWIDTH);
    htset=(1:HTYPE*LTYPE)';
    subRENTPROJ=zeros(NLENGTH,NWIDTH,LTYPE*HTYPE);
    pctbuildnew=zeros(LTYPE*HTYPE,TMAX);
    pctbuildold=zeros(LTYPE*HTYPE,TMAX);
    newhouses=zeros(LTYPE*HTYPE,TMAX);
    newbuild=zeros(LTYPE*HTYPE,TMAX);
    newacres=zeros(LTYPE*HTYPE,TMAX);
    oldhouses=zeros(LTYPE*HTYPE,TMAX);
    bidtot=zeros(LTYPE*HTYPE,TMAX);
    bidlevel=zeros(LTYPE*HTYPE,TMAX);
    newhouseset=zeros(LTYPE*HTYPE,TMAX);
    numnewhouses=zeros(1,TMAX);
    numoldhouses=zeros(1,TMAX);
    numnewacres=zeros(1,TMAX);
    Pdevbid=zeros(Nfarmers,TMAX);
    wtpland=zeros(Nfarmers,TMAX);
    wtahouse=zeros([],1);
    dist2vac=zeros(NLENGTH,NWIDTH);
    testdist2dev=zeros(NLENGTH,NWIDTH);
    subprofit=zeros(NLENGTH,NWIDTH);
    subrentproj=zeros(NLENGTH,NWIDTH,LTYPE*HTYPE);
    epsilon=zeros(1,TMAX);
    landdemand=zeros(Ndevelopers,TMAX);
    regionaldist=zeros(LTYPE*HTYPE,TMAX);
    regionalrent=zeros(LTYPE*HTYPE,TMAX);
    simlotrange=zeros(LTYPE*HTYPE,2);
    simlots_income=zeros(LTYPE*HTYPE,1);
    simlots_util=zeros(LTYPE*HTYPE,1);
    simlots_alpha=zeros(LTYPE*HTYPE,1);
    simlots_beta=zeros(LTYPE*HTYPE,1);
    simlots_gamma=zeros(LTYPE*HTYPE,1);
    vac_ccost=zeros(LTYPE*HTYPE,TMAX);
    vac_rent=zeros(LTYPE*HTYPE,TMAX);
    vac_land=zeros(TMAX,1);
    profits=zeros(LTYPE*HTYPE,TMAX);
    budget_lt=zeros(LTYPE*HTYPE,TMAX);
    carrycost=zeros(TMAX,1);
    BUDGET=zeros(TMAX,1);
    LANDBUDGET=zeros(1,TMAX);
    WTPMAP=zeros(NLENGTH,NWIDTH,TMAX);
    
    %%% Consumers %%%
    Income=zeros([],2);     %[Income Search_time]
    OutIncome=zeros([],2);
    RESTIME=zeros(NLENGTH,NWIDTH);
    avghousemp=zeros(1,TMAX);
    CONINFO=zeros([],3);
    NCON=zeros(WAGECLASS,TMAX);
    searchtimemin=2;
    searchtimemax=6;
    
    %%% Population %%%
    POP=zeros(1,TMAX);
    
    %%% Brokers %%%
    AVGUTIL=zeros(NLENGTH,NWIDTH);
    AVGINCOME=zeros(NLENGTH,NWIDTH);
    avg_income=zeros(NLENGTH,NWIDTH);
    avg_alpha=zeros(NLENGTH,NWIDTH);
    avg_beta=zeros(NLENGTH,NWIDTH);
    avg_gamma=zeros(NLENGTH,NWIDTH);
    avgstats=zeros(Nbrokers,4);%[avg_income avg_alpha avg_beta avg_gamma]
    houseinfo=zeros(LTYPE*HTYPE,6,Nbrokers,TMAX);      %[avg_price lotsize housesize #of_bidders %above_Pask #lots(lt)]
    EXPTHOUSE=zeros(NLENGTH,NWIDTH);
    MINBIDLEVEL=zeros(NLENGTH,NWIDTH);
    BIDLEVEL=zeros(NLENGTH,NWIDTH);
    bproj=zeros(LTYPE*HTYPE,NUMMODEL);
    brokerproj = zeros(LTYPE*HTYPE,NUMMODEL,Nbrokers);
    brokererror = zeros(LTYPE*HTYPE,NUMMODEL,Nbrokers,TMAX);
    brokerabserror = zeros(LTYPE*HTYPE,NUMMODEL,Nbrokers,TMAX);
    brokerbestSAVE=zeros(Nbrokers,LTYPE*HTYPE,TMAX);
    ibrokerbestSAVE=zeros(Nbrokers,LTYPE*HTYPE,TMAX);
    brokerprojSAVE=zeros(Nbrokers,LTYPE*HTYPE,TMAX);
    brokermodelSAVE=zeros(Nbrokers,LTYPE*HTYPE,TMAX);
    numlt=zeros(LTYPE*HTYPE,TMAX);
    newopenlots=zeros(LTYPE*HTYPE,TMAX);
    avgbrokervar=zeros(LTYPE*HTYPE,TMAX);                                                                       
    diffbrokererror=zeros(LTYPE*HTYPE,NUMMODEL,Nbrokers,TMAX);
	brokerbestdiffSAVE=zeros(Nbrokers,LTYPE*HTYPE,TMAX);
    brokerbestabsSAVE=zeros(Nbrokers,LTYPE*HTYPE,TMAX);
    
    %%%% RESULTS %%%%
    devcells=zeros(2,TMAX);         %[#dev_cells; pct_dev_land]
    vacstats=zeros(LTYPE*HTYPE,TMAX);
    avgrent=zeros(LTYPE*HTYPE,TMAX);
    vacrate=zeros(1,TMAX);
    nohouse=zeros(1,TMAX);
    oldincome=zeros(1,TMAX);
    consumerstats=zeros(5,TMAX);     %[Nconsumers; avg_income avg_util housemp income_out]
    vacantlots=zeros(1,TMAX);
    leftoverpop=zeros(1,TMAX);
    agrland=zeros(1,TMAX);
    BT=zeros(NLENGTH,NWIDTH,length(testtime));
    BL=zeros(NLENGTH,NWIDTH,length(testtime));
    Rvacland=zeros(NLENGTH,NWIDTH,length(testtime));
    Rrent=zeros(NLENGTH,NWIDTH,length(testtime));
    subRreturn=zeros(NLENGTH,NWIDTH);
    Rreturn=zeros(NLENGTH,NWIDTH,length(testtime));
    Rlottype=zeros(NLENGTH,NWIDTH,length(testtime));
    Rincome=zeros(NLENGTH,NWIDTH,length(testtime));
    Rbaselayer=zeros(NLENGTH,NWIDTH,length(testtime));
    Rpland=zeros(NLENGTH,NWIDTH,length(testtime));
    Rpop=zeros(1,length(testtime));
    Rvacrate=zeros(1,length(testtime));
    Rvaclots=zeros(1,length(testtime));
    Rnumlot=zeros(1,length(testtime));
    Rleftoverpop=zeros(1,length(testtime));
    idealset=zeros(LTYPE*HTYPE,length(testtime));
    profset=zeros(LTYPE*HTYPE,length(testtime));
    setupmap=zeros(NLENGTH,NWIDTH);
    utilgini=zeros(1,TMAX);
    incgini=zeros(1,TMAX);
    Exptrentdiff=zeros(NLENGTH,NWIDTH,TMAX);
    Avgexptdiff=zeros(LTYPE*HTYPE,TMAX);
    Newlottype=zeros(NLENGTH,NWIDTH,TMAX);
    Realreturn=zeros(NLENGTH,NWIDTH,TMAX);
    Realexpret=zeros(NLENGTH,NWIDTH,TMAX);
    Realavgret=zeros(LTYPE*HTYPE,TMAX);
    Realavgexptret=zeros(LTYPE*HTYPE,TMAX);
    Newbidlevel=zeros(NLENGTH,NWIDTH,TMAX);
    Avgnewbid=zeros(LTYPE*HTYPE,TMAX);
    
    numtotbids=zeros(LTYPE*HTYPE,TMAX);
    htincome=zeros(LTYPE*HTYPE,TMAX);
    htperyear=zeros(LTYPE*HTYPE,TMAX);
    Exptprofit=zeros(LTYPE*HTYPE,TMAX);
    Exptret=zeros(LTYPE*HTYPE,TMAX);
    
    totbrokerrecord=zeros(3,LTYPE*HTYPE,TMAX,Nbrokers);
    
    totfarmrecord=zeros(4,TMAX,Nfarmers);   %[wtaland; wtpland; landprojSAVE; landmodelSAVE]
    Allfarmdist=zeros(Nfarmers,1);
    Farmdist2dev=zeros(Nfarmers,TMAX);
    Dynltmap=zeros(NLENGTH,NWIDTH,TMAX);
    Dynrentmap=zeros(NLENGTH,NWIDTH,TMAX);
    Dynmaxretmap=zeros(NLENGTH,NWIDTH,TMAX);
    Dynretltmap=zeros(NLENGTH,NWIDTH,TMAX);
    Dynmaxprofmap=zeros(NLENGTH,NWIDTH,TMAX);
    Dynprofltmap=zeros(NLENGTH,NWIDTH,TMAX);
    ZONED=zeros(NLENGTH,NWIDTH);
    
    subdynlt=zeros(NLENGTH,NWIDTH);
    subdynrent=zeros(NLENGTH,NWIDTH);
    Bmodelmap=zeros(NLENGTH,NWIDTH,TMAX);
    Bprojmap=zeros(NLENGTH,NWIDTH,TMAX);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%    Load Reference Landscape    %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    savedState=stream.State;

    RefLandscape_CHALMS_base_v1_0_zone0
    
    stream.Substream=mrun;
    stream.State=savedState;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%    Model Runs    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%
    CHALMS_base_v1_0
    
    VARLAYER=VARLAYER+BASELAYER;
    
    ndate=datestr(date,'ddmmyy');
    fname=sprintf('results_CHALMS_base_v1_0_%d',mrun);
    
    save(fname,'consumerstats','vacstats','BT','Rvacland','Rrent','Rreturn',...
        'Rlottype','Rincome','Rbaselayer','Rpop','Rpland','Rvacrate','Rvaclots',...
        'Rnumlot','Rleftoverpop','avgrentdynms','rentdynstats','deltalots',...
        'farmsoldinfo','avgrent','avgfarmsize','stdfarmsize','DELTA',...
        'survivalrate','LOCWGHT','REGWGHT','PCTSEARCH','zeta','HIBETA',...
        'MIDBETA','LOWBETA','POPGROW','ccost','newhouseset','bidtot',...
        'meandisp','maxdevdist','utilgini','incgini','setupmap','zonedensity',...
        'pctutildiff','VARLAYER','vacrate','Farmstats','oldincome','Realreturn',...
        'Realavgret','Exptrentdiff','Avgexptdiff','Newlottype','htincome',...
        'numtotbids','totfarmrecord','htperyear','Newbidlevel','Dynltmap',...
        'Dynrentmap','totbrokerrecord','Farmdist2dev','Bprojmap','Bmodelmap',...
        'Dynplandmap','WTAlandmap','WTPlandmap','Landprojmap','Landmodmap',...
        'bidshare','buildshare','farmacreinfo','totfarmrecord','landdemand',...
        'Exptprofit','Exptret','Realexptret','Realavgexptret','idealset',...
        'profset','avgbrokervar','carrycost')
end
